import { Component, ViewChild, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { AlertController, IonList, LoadingController, ModalController, ToastController } from '@ionic/angular';

import { ScheduleFilterPage } from '../schedule-filter/schedule-filter';
import { ConferenceData } from '../../providers/conference-data';
import { UserData } from '../../providers/user-data';
import { API, graphqlOperation } from 'aws-amplify';
import { APIService } from '../../api.service';
import { TranslateService } from '@ngx-translate/core';

const listProducts = `
query listProducts {
  listCategorys {
    items {
      id
      name
      products {
        items {
          id
          name
          price
        }
      }
    }
  }
}
`;

const listReceipts = `
query listReceipts {
  listReceipts(filter: {
    finished: {
      eq: false
    }
  }) {
    items {
      id
      name
      subtotal
      payMethod
      receiptNo
      createdTimestamp
      paidTimestamp
      finished
    }
  }
}
`;

const listReceiptItems = `
query listReceipts {
  listReceipts(filter: {
    finished: {
      eq: false
    }
  }) {
    items {
      id
      name
      receiptItems {
        items {
          id
          name
          sku
          unit
          price
          quantity
          vat
          netto
          brutto
          varPercentage
        }
      }
    }
  }
}
`;

@Component({
  selector: 'page-schedule',
  templateUrl: 'schedule.html',
  styleUrls: ['./schedule.scss'],
})
export class SchedulePage implements OnInit {
  // Gets a reference to the list element
  @ViewChild('scheduleList') scheduleList: IonList;

  dayIndex = 0;
  queryText = '';
  segment = 'all';
  excludeTracks: any = [];
  shownSessions: any = [];
  groups: any = [];
  confDate: string;

  // Products loaded into the main frame
  categoryProducts: any = [];

  // Products loaded into the main frame
  receipts: any = [];
  selectedReceipt: any = {};

  // Cart containing receipt products
  cart: any = [];

  // Subtotal made from selected products
  subtotal = 0;

  // Currency
  appCurrency: string;

  // Count of items
  receiptItemsCount: 0;

  // Depending on selected segment whether to show cart or not
  showCart: boolean;

  translations: any = [];

  constructor(
    public alertCtrl: AlertController,
    public confData: ConferenceData,
    public loadingCtrl: LoadingController,
    public modalCtrl: ModalController,
    public router: Router,
    public toastCtrl: ToastController,
    public user: UserData,
    private apiService: APIService,
    public alertController: AlertController,
    private translate: TranslateService
  ) { }

  ngOnInit() {
    this.appCurrency = 'CZK';
    this.receiptItemsCount = 0;
    this.subtotal = 0;
    this.showCart = false;

    this.updateSchedule();

    this.getProducts();

    this.getReceipts();

    this.getTranslations();
  }

  getTranslations(): any {
    this.translate.get(['paidConfirmation']).subscribe((res: []) => {
      console.log('Translations loaded.');
      this.translations = res;
    });
  }

  getProducts(): any {
    console.log('loading products..');
    const catQuery = API.graphql(graphqlOperation(listProducts)) as Promise<any>;

    catQuery.then(res => {
      this.categoryProducts = res.data.listCategorys.items;
    });
  }

  getReceipts(): any {
    console.log('loading receipts..');
    const receiptsQuery = API.graphql(graphqlOperation(listReceipts)) as Promise<any>;

    receiptsQuery.then(res => {
      this.receipts = res.data.listReceipts.items;
      this.selectedReceipt = this.receipts[0];

      console.log(this.receipts);
      if (this.receipts.length === 0) {
        // No receipt is created, in order to add products
        // It is needed to create a new receipt
        this.createReceipt();
      } else {
        console.log('Current receipt ID: ' + this.selectedReceipt.id);
        const itemsQuery = API.graphql(graphqlOperation(listReceiptItems)) as Promise<any>;
        itemsQuery.then(reply => {
          console.log('Loading receipt items..');
          const tmpReply = reply.data.listReceipts.items;
          if (tmpReply.length > 0) {
            const tmp = tmpReply[0].receiptItems.items;
            this.cart = tmp;
            console.log(this.cart);

            // Update item count and receipt subtotal
            this.updateItemCount();
            this.subtotal = this.getTotal();
          }
        });
      }
    });
  }

  createReceipt(): any {
    const ts = new Date().getTime().toString();
    this.apiService.CreateReceipt({
      name: 'Receipt init',
      subtotal: 0.00,
      finished: false,
      createdTimestamp: ts
    }).then(res => {
      console.log(res);
      console.log('receipt created!');
      this.selectedReceipt = res;
      console.log('Assigned new receipt ID: ' + this.selectedReceipt.id);
    });
  }

  addToReceipt(product): any {
    console.log('adding to receipt..');
    if (!this.selectedReceipt) {
      // Check if we have a receipt
      console.log('No receipt available!');
      return;
    }

    this.apiService.GetProduct(product.id).then(prod => {
      this.apiService.CreateItem({
        itemReceiptId: this.selectedReceipt.id,
        itemProductId: prod.id,
        name: prod.name,
        sku: prod.sku,
        unit: 'ks',
        price: prod.price,
        quantity: 1,
        vat: 0,
        netto: prod.price,
        brutto: prod.price,
        varPercentage: 21
      }).then(res => {
        this.receiptItemsCount += 1;
        this.subtotal += product.price;
        this.cart.push(product);
        console.log(this.cart);
      });
    });
  }

  updateItemCount(): any {
    this.receiptItemsCount = this.cart.length;
  }

  camera(): any {
    console.log('QR camera start..');
  }

  cancelReceipt(): any {
    console.log('Cancelling receipt..');
    this.presentAlertConfirm();
  }

  payReceipt(): any {
    const ts = new Date().getTime().toString();
    console.log('Paying receipt..');
    this.apiService.UpdateReceipt({
      id: this.selectedReceipt.id,
      finished: true,
      paidTimestamp: ts,
      subtotal: this.getTotal(),
    }).then(res => {
      console.log('Receipt paid.');
      this.showPaidToast();

      this.clearCart();
      this.createReceipt();

      this.showCart = false;
      this.segment = 'all';
    });
  }

  clearCart(): any {
    this.cart = [];
    this.receiptItemsCount = 0;
    this.subtotal = 0;
    console.log('Cart cleared.');
  }

  getTotal(): number {
    let tmp = 0;

    for (const item of this.cart) {
      tmp += item.price;
    }

    console.log('Calculated total: ' + tmp);
    return tmp;
  }

  updateSchedule() {
    // Close any open sliding items when the schedule updates
    if (this.scheduleList) {
      this.scheduleList.closeSlidingItems();
    }

    this.confData.getTimeline(this.dayIndex, this.queryText, this.excludeTracks, this.segment).subscribe((data: any) => {
      this.shownSessions = data.shownSessions;
      this.groups = data.groups;
    });
  }

  changeSegment(seg) {
    console.log('segment: ' + seg);
    if (seg === 'cart') {
      this.showCart = true;
    } else {
      this.showCart = false;
    }
  }

  async showPaidToast() {
    const toast = await this.toastCtrl.create({
      message: this.translations.paidConfirmation,
      duration: 3000
    });
    await toast.present();
  }

  async presentAlertConfirm() {
    const alert = await this.alertController.create({
      header: 'Confirm!',
      message: 'Message <strong>text</strong>!!!',
      buttons: [
        {
          text: 'Cancel',
          role: 'cancel',
          cssClass: 'secondary',
          handler: (blah) => {
            console.log('Confirm Cancel: blah');
          }
        }, {
          text: 'Okay',
          handler: () => {
            console.log('Confirm Okay');
          }
        }
      ]
    });

    await alert.present();
  }

  async presentFilter() {
    const modal = await this.modalCtrl.create({
      component: ScheduleFilterPage,
      componentProps: { excludedTracks: this.excludeTracks }
    });
    await modal.present();

    const { data } = await modal.onWillDismiss();
    if (data) {
      this.excludeTracks = data;
      this.updateSchedule();
    }
  }

  async addFavorite(slidingItem: HTMLIonItemSlidingElement, sessionData: any) {
    if (this.user.hasFavorite(sessionData.name)) {
      // woops, they already favorited it! What shall we do!?
      // prompt them to remove it
      this.removeFavorite(slidingItem, sessionData, 'Favorite already added');
    } else {
      // remember this session as a user favorite
      this.user.addFavorite(sessionData.name);

      // create an alert instance
      const alert = await this.alertCtrl.create({
        header: 'Favorite Added',
        buttons: [{
          text: 'OK',
          handler: () => {
            // close the sliding item
            slidingItem.close();
          }
        }]
      });
      // now present the alert on top of all other content
      await alert.present();
    }

  }

  async removeFavorite(slidingItem: HTMLIonItemSlidingElement, sessionData: any, title: string) {
    const alert = await this.alertCtrl.create({
      header: title,
      message: 'Would you like to remove this session from your favorites?',
      buttons: [
        {
          text: 'Cancel',
          handler: () => {
            // they clicked the cancel button, do not remove the session
            // close the sliding item and hide the option buttons
            slidingItem.close();
          }
        },
        {
          text: 'Remove',
          handler: () => {
            // they want to remove this session from their favorites
            this.user.removeFavorite(sessionData.name);
            this.updateSchedule();

            // close the sliding item and hide the option buttons
            slidingItem.close();
          }
        }
      ]
    });
    // now present the alert on top of all other content
    await alert.present();
  }

  async openSocial(network: string, fab: HTMLIonFabElement) {
    const loading = await this.loadingCtrl.create({
      message: `Posting to ${network}`,
      duration: (Math.random() * 1000) + 500
    });
    await loading.present();
    await loading.onWillDismiss();
    fab.close();
  }
}
